//*****************************************************************************************
// Truevision - a 3d modeler for gnome and povray
//
// interface.cc
//
// Vincent LE PRINCE <vincentleprince@users.sourceforge.net>
// Copyright (C) 2000-2001 Vincent LE PRINCE
// This file is part of the TRUEVISION Package

//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */ 
//*******************************************************************************************
#include <iostream>
#include "config.h"
#include "include/interface.h"
#include "include/preferences.h"
#include "include/scene.h"
#include "include/objectlist.h"
#include "include/povfe.h"
#include "include/tvio.h"
#include "include/undo.h"
#include "include/objectlist.h"

#ifdef HAVE_TV_PYTHON
#include "include/pyengine.h"
#endif

//***************************************************
// Constructeur
//***************************************************
Interface::Interface( app_objs *appref )
{
// App gnome
app_ref = appref;
app_ref->interf = (app_object*)this;

win = gnome_app_new( PACKAGE, APP_NAME );
gtk_signal_connect( GTK_OBJECT(win), "delete_event", GTK_SIGNAL_FUNC(app_exit), NULL );
vbox = gtk_hbox_new( FALSE, 1 );
gnome_app_set_contents( GNOME_APP(win), vbox );
gtk_container_set_border_width( GTK_CONTAINER(win), 1 );
gtk_container_set_border_width( GTK_CONTAINER(vbox), 0 );
paned = gtk_hpaned_new();
gtk_box_pack_start( GTK_BOX(vbox), paned, TRUE, TRUE, 0 );

//gchar *fname = gnome_program_locate_file( app_ref->app, GNOME_FILE_DOMAIN_PIXMAP, "truevision.png", FALSE, NULL );
gchar *fname = tv_get_pixmap( "truevision.png" );
GdkPixbuf *buffer = gdk_pixbuf_new_from_file( fname, NULL );
gtk_window_set_icon( GTK_WINDOW(win), buffer );
g_free( fname );	
	
accels = gtk_accel_group_new();
gtk_window_add_accel_group( GTK_WINDOW(win), accels );
gtk_widget_realize( win );

SCENE_DEF
scene->set_win_title();

// Prfrences
PREF_DEF
if ( pref->save_geo->value() )
	{
	if ( pref->win_xpos->value() < 0 ) pref->win_xpos->set(0);
	if ( pref->win_ypos->value() < 0 ) pref->win_ypos->set(0);	
	gtk_widget_set_uposition( win, pref->win_xpos->value(), pref->win_ypos->value() );
	gtk_window_resize( GTK_WINDOW(win), pref->win_xsize->value(), pref->win_ysize->value() );
	gtk_paned_set_position( GTK_PANED(paned), pref->pan_pos->value() );
	}
	
// Vue 3d
vmanager = new ViewManager( app_ref, paned );
	
// Side bar
GtkWidget *box = gtk_vbox_new( FALSE, 0 );
gtk_paned_add2( GTK_PANED(paned), box );
hide_button = gtk_button_new();
gtk_box_pack_start( GTK_BOX(box), hide_button, FALSE, TRUE, 0 );
gtk_signal_connect( GTK_OBJECT(hide_button), "clicked", GTK_SIGNAL_FUNC(sign_interf_hide_panel), this );
side_panel_box = gtk_vbox_new( FALSE, 1 );
gtk_box_pack_start( GTK_BOX(box), side_panel_box, TRUE, TRUE, 0 );
	// PropertiesEditor
	panel = new PropPanel( app_ref, side_panel_box );
	
	// App bar
	appbar = gnome_appbar_new( TRUE, TRUE, GNOME_PREFERENCES_NEVER );
	gtk_box_pack_start( GTK_BOX(side_panel_box), appbar, FALSE, TRUE, 0 );

	
// Toolbar
toolbar = new ToolBar( app_ref, win );

// MENUS
// Export
static GnomeUIInfo export_menu[] = {
	{ GNOME_APP_UI_ITEM, N_("To povray..."), NULL, (void*)sign_export_to_povray, app_ref->scene, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, 0, (GdkModifierType)0, NULL },
	GNOMEUIINFO_END
};

// Recent
static GnomeUIInfo recent_menu[] = {
	GNOMEUIINFO_END
};

//OBJLIST_DEF
// Menu FILE
static GnomeUIInfo file_menu[] = { 
	GNOMEUIINFO_MENU_NEW_ITEM( N_("New Scene"), NULL, sign_scene_new, app_ref->scene ),
	GNOMEUIINFO_MENU_OPEN_ITEM(  sign_scene_load, app_ref->scene ),	
	GNOMEUIINFO_MENU_SAVE_ITEM(sign_scene_save, app_ref->scene ),
	GNOMEUIINFO_MENU_SAVE_AS_ITEM(sign_scene_save_as, app_ref->scene ),	
	{ GNOME_APP_UI_SUBTREE, N_("Export..."), NULL, export_menu, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CONVERT, 0, (GdkModifierType)0, NULL },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_SUBTREE, N_("Recent files..."), NULL, recent_menu, NULL, NULL,GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, 0, (GdkModifierType)0, NULL },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Load object..."), NULL, (void*)sign_objlist_load_object, app_ref->obj3dlist, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, 0, (GdkModifierType)0, NULL },
	{ GNOME_APP_UI_ITEM, N_("Save object..."), NULL,  (void*)sign_objlist_save_object, app_ref->obj3dlist, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS, 0, (GdkModifierType)0, NULL },
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM(app_exit, NULL), 
	GNOMEUIINFO_END
	};
// Menu EDIT
static GnomeUIInfo edit_menu[] = {
	GNOMEUIINFO_MENU_UNDO_ITEM( (void*)sign_undoman_undo, app_ref->undoman ),
	GNOMEUIINFO_MENU_REDO_ITEM( (void*)sign_undoman_redo, app_ref->undoman ),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_CUT_ITEM( (void*)sign_objlist_cut_object, app_ref->obj3dlist ),
	GNOMEUIINFO_MENU_COPY_ITEM( (void*)sign_objlist_copy_object, app_ref->obj3dlist ),
	GNOMEUIINFO_MENU_PASTE_ITEM( (void*)sign_objlist_paste_object, app_ref->obj3dlist ),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_PREFERENCES_ITEM(pref_dlg, pref ),	
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Render"), NULL, (void*)sign_render_box, app_ref->povfe, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXEC, 0, (GdkModifierType)0, NULL },	
	{ GNOME_APP_UI_ITEM, N_("Render options"), NULL, (void*)sign_options_box, app_ref->povfe, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 0, (GdkModifierType)0, NULL },	
	GNOMEUIINFO_END
};

// Menu SCRIPT
#ifdef HAVE_TV_PYTHON 
static GnomeUIInfo script_menu[] = {
	 {GNOME_APP_UI_ITEM, N_("Execute Script"), NULL, (void*)sign_execute_script, app_ref->python_engine, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXEC, 0, (GdkModifierType)0, NULL },
	GNOMEUIINFO_END 
};
#endif
 
// Menu HELP
static GnomeUIInfo help_menu[] = { 
	GNOMEUIINFO_HELP ((char *)"truevision"),
	{ GNOME_APP_UI_ITEM, N_("Help on..."), NULL, (void*)sign_help_on_obj, app_ref->obj3dlist,  NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, (GdkModifierType)0, NULL },	
	{ GNOME_APP_UI_ITEM, N_("Truevision homepage"), NULL, (void*)display_homepage, NULL,  NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, (GdkModifierType)0, NULL },	
	GNOMEUIINFO_MENU_ABOUT_ITEM(about, NULL ),
	GNOMEUIINFO_END 
	};
// Barre de Menus
static GnomeUIInfo main_menu[] = { 
	GNOMEUIINFO_MENU_FILE_TREE(file_menu),
	GNOMEUIINFO_MENU_EDIT_TREE(edit_menu),
#ifdef HAVE_TV_PYTHON	
	GNOMEUIINFO_SUBTREE(N_("Scripts"), script_menu),
#endif 
	GNOMEUIINFO_MENU_HELP_TREE(help_menu),
	GNOMEUIINFO_END 
	};
gnome_app_create_menus( GNOME_APP(win), main_menu );

gtk_widget_show_all( win );
//if ( pref->save_geo->value() ) 
	//gdk_window_resize( win->window, (gint)(pref->win_xsize->value()), (gint)(pref->win_ysize->value()) );
	//cout << "\nSize = " << pref->win_xsize->value() << " x " << pref->win_ysize->value();
//else gdk_window_resize( win->window, 500, 500 );

GList *menu = gtk_container_get_children( GTK_CONTAINER(GNOME_APP(win)->menubar) );	
GtkMenu *filemenu = (GtkMenu*)gtk_menu_item_get_submenu( GTK_MENU_ITEM(menu->data) );
menu = gtk_container_get_children( GTK_CONTAINER(filemenu) );		
menu = g_list_nth( menu, 6 );
recent_files = (GtkMenuItem*)menu->data;

set_recent_files();	
	
UNDO_DEF
undoman->set_undo_menu_widgets(  edit_menu[0].widget,  edit_menu[1].widget );
scene->set_save_menu_widget( file_menu[2].widget );
gtk_widget_set_sensitive(  file_menu[2].widget, FALSE );
OBJLIST_DEF	
objlist->set_help_on_wid( help_menu[1].widget );
vmanager->set_pointer_mode( TV_PMODE_TRACKBALL );

// Set panel state
if ( pref->pan_hidden->value() ) 
	{
	pref->pan_hidden->set( false );
	hide_panel();
	}
}

void Interface::hide_panel( bool change )
{
PREF_DEF
bool panel_hidden = pref->pan_hidden->value();
int panel_size = pref->pan_pos->value();
if ( panel_hidden ) 
	{
	gtk_widget_show_all( side_panel_box );
	gtk_paned_set_position( GTK_PANED(paned), panel_size );
	gtk_widget_set_usize( hide_button, -1, 8 );
	}
else 
	{
	int hei = side_panel_box->allocation.height;
	panel_size = gtk_paned_get_position( GTK_PANED(paned) );
	gtk_widget_hide_all( side_panel_box );
	gtk_paned_set_position( GTK_PANED(paned), win->allocation.width-16 );
	gtk_widget_set_usize( hide_button, 8, hei );
	}
if ( change ) pref->pan_hidden->set( ! panel_hidden );
pref->pan_pos->set( panel_size );
}

void Interface::jitter()
{
int size = gtk_paned_get_position( GTK_PANED(paned) );
gtk_paned_set_position( GTK_PANED(paned), size-1 );
while ( gtk_events_pending() ) gtk_main_iteration();
gtk_paned_set_position( GTK_PANED(paned), size );	
}

//*******************************************************
// Prfrences
//*******************************************************
void Interface::flush_pref()
{
PREF_DEF
gint x,y;
gdk_window_get_root_origin( win->window, &x, &y );
pref->win_xpos->set( x );
pref->win_ypos->set( y );
pref->win_xsize->set( win->allocation.width );
pref->win_ysize->set( win->allocation.height );
if ( ! pref->pan_hidden->value() ) pref->pan_pos->set( gtk_paned_get_position( GTK_PANED(paned) ) );
}



//********************************************************
// Save
//********************************************************
void Interface::save( ofstream & file )
{
file << "\nINTERFACE{";
vmanager->save( file );

file << "\n}";
}


void Interface::clear()
{
vmanager->clear();
}


bool Interface::load( ifstream & file, char * ltag )
{
if ( strcmp( ltag, "INTERFACE" ) ) return false;

char * tag = NULL;
do {
	tag = tvio_get_next_tag( file );
	if ( tag == NULL ) break;
	if ( vmanager->load( file, tag ) ) continue;
		
	tvio_skip_section(file );
	} while ( tag != NULL );
return true;
}


//******************************************************
// Set recent files
//******************************************************
void Interface::set_recent_files()
{
PREF_DEF
gtk_menu_item_remove_submenu( recent_files );
GtkMenu *menu = (GtkMenu*)gtk_menu_new();
gtk_menu_item_set_submenu( recent_files, (GtkWidget*)menu );
int size = pref->recent_files->get_size();
for ( register int i = 0 ; i < size ; i++ )
	{
	char *str = pref->recent_files->value(i);
	if ( str == NULL ) break;
	recent_files_widgets[i] = gtk_menu_item_new_with_label( str );
	gtk_signal_connect( GTK_OBJECT(recent_files_widgets[i]), "activate", GTK_SIGNAL_FUNC(sign_interface_recent_file), this );
	gtk_menu_append( menu,  recent_files_widgets[i] );
	}
gtk_widget_show_all( (GtkWidget*)menu );
}


void Interface::recent_file_push( char *filename )
{
PREF_DEF
pref->recent_files->push_string_unique( filename );
set_recent_files();
}

void Interface::recent_file_pop( char *filename )
{
PREF_DEF
if ( pref->recent_files->pop_string( filename ) )
	set_recent_files();
}


void Interface::load_recent_file( GtkWidget *wid )
{
int num = -1;
for ( int i = 0 ; i < 10 ; i++ ) if ( wid == recent_files_widgets[i] ) { num = i; break; }
SCENE_DEF
PREF_DEF
if ( num != -1 ) 
	{
	scene->load( pref->recent_files->value( num ) );
	pref->recent_files->reput_on_top( num );
	set_recent_files();
	}
}



void Interface::set_hourglass( bool set )
{
if ( set == true )
	{
	GdkCursor *pointeur = gdk_cursor_new( GDK_WATCH );
	gdk_window_set_cursor( win->window, pointeur );
	gdk_cursor_destroy( pointeur );
	vmanager->unset_cursor();
	}
else
	{
	//GdkCursor *pointeur = gdk_cursor_new( GDK_ARROW );
	gdk_window_set_cursor( win->window, NULL );
	//gdk_cursor_destroy( pointeur );
	vmanager->set_pointer_mode();
	}
}
